// chflashcard.cc

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/param.h>
#include <getopt.h>

extern "C" {
#include "libsx.h"	
}
extern "C" {	
#include "freq.h"	
}
#include "datos.h"
#include "tarjeta.h"
#include "opciones.h"
#include "global.h"

#define MAX_FONT 255

char G_HanziFont[MAX_FONT];
char G_TextFont[MAX_FONT];
static char G_InitialPath[MAXPATHLEN];

static Datos d;

static int init_display(int argc, char **argv, void *data);
static void Flashcard(Datos *d, int tipo);
static void copiarString(char *d,char *s,unsigned int l);
static void formaDeUso();
static void leerOpciones(int argc, char *argv[]);

/* callback protos */
static void quit(Widget w, void *data);
static void t(Widget w, void *data);
static void load(Widget w, void *data);
static void o(Widget w, void *data);
static void Opcion(Datos *d, int tipo);

int main(int argc, char **argv)
{
  leerOpciones(argc,argv);
  
  argc = init_display(argc, argv, NULL);  /* setup the display */
  if (argc == 0)
    exit(0);
  MainLoop();                                /* go right into the main loop */
}


/* This function sets up the display.  For any kind of a real program, 
 * you'll probably want to save the values returned by the MakeXXX calls
 * so that you have a way to refer to the display objects you have 
 * created (like if you have more than one drawing area, and want to
 * draw into both of them).
 */

static int init_display(int argc, char **argv, void *data)
{
  Widget w[3];

  argc = OpenDisplay(argc, argv);
  if (argc == FALSE)
    return argc;

  w[0]  = MakeMenu(S_1);
          MakeMenuItem(w[0],S_2,load,data);
          MakeMenuItem(w[0],S_3,quit,data);
  w[1]  = MakeMenu(S_4);
          MakeMenuItem(w[1],S_5,t,(void *)1);
          MakeMenuItem(w[1],S_6,t,(void *)2);
          MakeMenuItem(w[1],S_7,t,(void *)3);
  w[2]  = MakeMenu(S_8);
          MakeMenuItem(w[2],S_9,o,(void *)1);
          MakeMenuItem(w[2],S_10,o,(void *)2);
          MakeMenuItem(w[2],S_11,o,(void *)3);
          MakeMenuItem(w[2],S_12,o,(void *)4);

  SetWidgetPos(w[1], PLACE_RIGHT, w[0], NO_CARE, NULL);
  SetWidgetPos(w[2], PLACE_RIGHT, w[1], NO_CARE, NULL);

  ShowDisplay();
  
  GetStandardColors();
  MainLoop();
  return argc;
}

/*
 * load() - Callback function for the load button.  This just calls
 *          GetFile() to get a file name
 */
static void load(Widget, void *)
{
  char *fname;
  int l;

  fname = GetFile(G_InitialPath,".tst");
  if(!fname) return;
  l=strlen(fname);
  if(l>4&&fname[l-1]=='t'&&fname[l-2]=='s'&&fname[l-3]=='t'&&fname[l-4]=='.')
  {
    d.leerFichero(fname);
  }
}

/*
 * quit() - Callback funtion for the quit button.
 */
static void quit(Widget , void *)
{
  exit(0);
}


static void t(Widget , void *data)
{
  if (d.numero()>0)
    Flashcard(&d,(int)data);
  else
    GetYesNo(S_13);
}

static void Flashcard(Datos *d, int tipo)
{
  Tarjeta t(d,tipo);

  t.ejecutar();
}

static void o(Widget , void *data)
{
  if (d.numero()>0)
    Opcion(&d,(int)data);
  else
    GetYesNo(S_13);
}

static void Opcion(Datos *d, int tipo)
{
  Opciones o(d,tipo);

  o.ejecutar();
}

static void copiarString(char *d,char *s,unsigned int l)
{
  if(strlen(s)<=l)
   strcpy(d,s);
}


static void formaDeUso()
{
  fprintf(stderr,"Usage: \n");
  fprintf(stderr,"chflashcard [options]\n");
  fprintf(stderr,"options:\n");
  fprintf(stderr,"-c <Chinese font>         (ex: -c hku-ch24\n");
  fprintf(stderr,"-a <Ascii font>           (ex: -a 8x16)\n");
  fprintf(stderr,"-p <Initial path>         (ex: -p /usr/local/flashcards\n");
  exit(0);
}

static void leerOpciones(int argc, char *argv[])
{
  int c;

   strcpy(G_HanziFont,DEFAULT_HANZI_FONT);
   strcpy(G_TextFont,DEFAULT_TEXT_FONT);
   strcpy(G_InitialPath,PATH);
   while(1)
   {
     c=getopt(argc,argv,"c:a:p:h?");
     if(c == -1) break;

     switch(c)
     {
       case 'c': copiarString(G_HanziFont,optarg,MAX_FONT);break;
       case 'a': copiarString(G_TextFont,optarg,MAX_FONT); break;
       case 'p': copiarString(G_InitialPath,optarg,MAXPATHLEN); break;
       case 'h':
       case '?': formaDeUso(); break;
     }

   }
}
